<?php
namespace KnitPay\Gateways\Stripe;

use Pronamic\WordPress\Pay\Core\Gateway as Core_Gateway;
use Pronamic\WordPress\Pay\Payments\Payment;
use Pronamic\WordPress\Pay\Payments\PaymentStatus;
use Pronamic\WordPress\Pay\Payments\FailureReason;
use WP_Error;


/**
 * Title: Stripe Gateway
 * Copyright: 2020-2021 Knit Pay
 *
 * @author Knit Pay
 * @version 1.0.0
 * @since 3.1.0
 */
class Gateway extends Core_Gateway {


	const NAME = 'stripe';

	/**
	 * Constructs and initializes an Stripe gateway
	 *
	 * @param Config $config
	 *            Config.
	 */
	public function __construct( Config $config ) {
		parent::__construct( $config );

		$this->set_method( self::METHOD_HTML_FORM );

		// Supported features.
		$this->supports = array(
			'payment_status_request',
		);

		\Stripe\Stripe::setAppInfo( 'Knit Pay', null, 'https://www.knitpay.org/' );
	}

	/**
	 * Get supported payment methods
	 *
	 * @see Core_Gateway::get_supported_payment_methods()
	 */
	public function get_supported_payment_methods() {
		return array(
			PaymentMethods::BANK_TRANSFER,
			PaymentMethods::CREDIT_CARD,
			PaymentMethods::MAESTRO,
			PaymentMethods::STRIPE,
			PaymentMethods::SOFORT,
		);
	}

	/**
	 * Get available payment methods.
	 *
	 * @return array<int, string>
	 * @see Core_Gateway::get_available_payment_methods()
	 */
	public function get_available_payment_methods() {
		return $this->get_supported_payment_methods();
	}

	/**
	 * Start.
	 *
	 * @see Core_Gateway::start()
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function start( Payment $payment ) {
		$this->stripe_session_id = $payment->get_meta( 'stripe_session_id' );

		// Return if session_id already exists for this payments
		if ( $this->stripe_session_id ) {
			return;
		}

		$secret_key              = $this->config->secret_key;
		$stripe_payment_currency = $this->config->payment_currency;
		$exchange_rate           = $this->config->exchange_rate;

		$payment_currency = $payment->get_total_amount()->get_currency()->get_alphabetic_code();
		$payment_amount   = $payment->get_total_amount()->format() * 100;
		$customer         = $payment->get_customer();

		if ( ! empty( $stripe_payment_currency ) && $stripe_payment_currency !== $payment_currency ) {
			$payment_currency = $stripe_payment_currency;
			$payment_amount   = $exchange_rate * $payment_amount;
		}
		$payment_amount = round( $payment_amount );

		$stripe = new \Stripe\StripeClient(
			$secret_key
		);

		$payment_method_types[] = 'card';
		if ( PaymentMethods::SOFORT === $payment->get_method() ) {
			$payment_method_types = array( 'sofort' );
		}

		$session_data = array(
			'success_url'          => $payment->get_return_url(),
			'client_reference_id'  => $payment->id,
			'customer_email'       => $customer->get_email(),
			'cancel_url'           => $payment->get_return_url(),
			'payment_method_types' => $payment_method_types,
			'line_items'           => array(
				array(
					'price_data' => array(
						'currency'     => $payment_currency,
						'unit_amount'  => $payment_amount,
						'product_data' => array(
							'name' => $payment->get_description(),
						),
					),
					'quantity'   => 1,
				),
			),
			'mode'                 => 'payment',
			'metadata'             => array( 'knitpay_payment_id' => $payment->id ),
		);

		// TODO: improve  line items
		$stripe_session = $stripe->checkout->sessions->create( $session_data );

		$payment->set_meta( 'stripe_session_id', $stripe_session->id );
		$this->stripe_session_id = $stripe_session->id;

		$payment->set_transaction_id( $stripe_session->payment_intent );
		$payment->set_action_url( $payment->get_pay_redirect_url() );
	}

	/**
	 * Get form HTML.
	 *
	 * @param Payment $payment     Payment to get form HTML for.
	 * @param bool    $auto_submit Flag to auto submit.
	 * @return string
	 * @throws \Exception When payment action URL is empty.
	 */
	public function get_form_html( Payment $payment, $auto_submit = false ) {

		$publishable_key         = $this->config->publishable_key;
		$this->stripe_session_id = $payment->get_meta( 'stripe_session_id' );

		$form_inner = $this->get_output_html( $payment );
		$form_inner = '<button class="pronamic-pay-btn" id="checkout-button">Checkout</button>';

		$form_inner .= '<script src="https://js.stripe.com/v3/">

        </script><script type="text/javascript">
	    // Create an instance of the Stripe object with your publishable API key
	    var stripe = Stripe("' . $publishable_key . '");
	    var checkoutButton = document.getElementById("checkout-button");
        result = stripe.redirectToCheckout({sessionId: "' . $this->stripe_session_id . '"});
	    
	    </script>';
		return $form_inner;

	}

	/**
	 * Update status of the specified payment.
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function update_status( Payment $payment ) {
		if ( PaymentStatus::SUCCESS === $payment->get_status() ) {
			return;
		}

		$secret_key = $this->config->secret_key;
		$stripe     = new \Stripe\StripeClient( $secret_key );

		$stripe_paymentIntents = $stripe->paymentIntents->retrieve( $payment->get_transaction_id(), array() );

		// Return if payment not attempted yet.
		if ( empty( $stripe_paymentIntents->charges->total_count ) ) {
			return;
		}

		$payment->set_status( Statuses::transform( $stripe_paymentIntents->status ) );
		$note = 'Stripe Charge ID: ' . $stripe_paymentIntents->charges->data[0]->id . '<br>Stripe Payment Status: ' . $stripe_paymentIntents->status;

		if ( isset( $stripe_paymentIntents->last_payment_error ) ) {
			$failure_reason = new FailureReason();
			$failure_reason->set_message( $stripe_paymentIntents->last_payment_error->message );
			$failure_reason->set_code( $stripe_paymentIntents->last_payment_error->code );
			$payment->set_failure_reason( $failure_reason );
			$payment->set_status( PaymentStatus::FAILURE );
			$note .= '<br>Error Message: ' . $stripe_paymentIntents->last_payment_error->message;
		}
		$payment->add_note( $note );

	}
}
